@@ -103,16 +103,12 @@ fill_pcx_context(struct pcx_context *context)
 static void
 free_pcx_context(struct pcx_context *context, gboolean unref_pixbuf)
 {
-	if(context->header)
-		g_free(context->header);
-	if(context->buf)
-		g_free(context->buf);
+	g_free(context->header);
+	g_free(context->buf);
 	if(unref_pixbuf && context->pixbuf)
 		g_object_unref(context->pixbuf);
-	if(context->line)
-		g_free(context->line);
-	if(context->p_data)
-		g_free(context->p_data);
+	g_free(context->line);
+	g_free(context->p_data);
 
 	g_free(context);
 }
@@ -236,14 +232,12 @@ pcx_chop_context_buf(struct pcx_context *context, guint size)
 {
 	guint i, j;
 
-	if(size > context->buf_pos)
+	if (size > context->buf_pos)
 		return FALSE;
-	else if(size < 0)
-		return FALSE;
-	else if(size == 0)
+	else if (size == 0)
 		return TRUE;
 
-	for(i = 0, j = size; j < context->buf_pos; i++, j++)
+	for (i = 0, j = size; j < context->buf_pos; i++, j++)
 		context->buf[i] = context->buf[j];
 
 	context->buf_pos -= size;
@@ -353,7 +347,7 @@ pcx_increment_load_data_1(struct pcx_context *context)
 		}
 
 		if(context->updated_func)
-			context->updated_func(context->pixbuf, 0, 0, context->width, context->height, context->user_data);
+			context->updated_func(context->pixbuf, 0, context->current_line, context->width, 1, context->user_data);
 
 		context->current_line++;
 
@@ -493,7 +487,7 @@ pcx_load_palette_8(struct pcx_context *context)
 				}
 
 				if(context->updated_func)
-					context->updated_func(context->pixbuf, 0, context->current_line, context->width, 1, context->user_data);
+					context->updated_func(context->pixbuf, 0, i, context->width, 1, context->user_data);
 			}
 
 #ifdef PCX_DEBUG
@@ -629,7 +623,7 @@ gdk_pixbuf__pcx_load_increment(gpointer data, const guchar *buf, guint size,
 				case 4:
 				case 8:
 					if(context->num_planes != 1) {
-						g_set_error(error, GDK_PIXBUF_ERROR, GDK_PIXBUF_ERROR_UNKNOWN_TYPE, _("Image has unsupported number of %d-bit planes"), context->bpp);
+					  g_set_error(error, GDK_PIXBUF_ERROR, GDK_PIXBUF_ERROR_UNKNOWN_TYPE, _("Image has unsupported number of %d-bit planes"), (gint)context->bpp);
 						return FALSE;
 					}
 					break;
@@ -725,6 +719,12 @@ gdk_pixbuf__pcx_stop_load(gpointer data, GError **error)
 	return TRUE;
 }
 
+#ifndef INCLUDE_pcx
+#define MODULE_ENTRY(type,function) function
+#else
+#define MODULE_ENTRY(type,function) _gdk_pixbuf__ ## type ## _ ## function
+#endif
+
 void
 MODULE_ENTRY (pcx, fill_vtable) (GdkPixbufModule *module)
 {
