@@ -28,7 +28,10 @@
 #include <string.h>
 #include "gtkframe.h"
 #include "gtklabel.h"
+#include "gtkprivate.h"
 #include "gtkintl.h"
+#include "gtkbuildable.h"
+#include "gtkalias.h"
 
 #define LABEL_PAD 1
 #define LABEL_SIDE_PAD 2
@@ -43,9 +46,6 @@ enum {
   PROP_LABEL_WIDGET
 };
 
-
-static void gtk_frame_class_init    (GtkFrameClass  *klass);
-static void gtk_frame_init          (GtkFrame       *frame);
 static void gtk_frame_set_property (GObject      *object,
 				    guint         param_id,
 				    const GValue *value,
@@ -74,35 +74,16 @@ static void gtk_frame_compute_child_allocation      (GtkFrame      *frame,
 static void gtk_frame_real_compute_child_allocation (GtkFrame      *frame,
 						     GtkAllocation *child_allocation);
 
-static GtkBinClass *parent_class = NULL;
+/* GtkBuildable */
+static void gtk_frame_buildable_init                (GtkBuildableIface *iface);
+static void gtk_frame_buildable_add                 (GtkBuildable *buildable,
+						     GtkBuilder   *builder,
+						     GObject      *child,
+						     const gchar  *type);
 
-
-GType
-gtk_frame_get_type (void)
-{
-  static GType frame_type = 0;
-
-  if (!frame_type)
-    {
-      static const GTypeInfo frame_info =
-      {
-	sizeof (GtkFrameClass),
-	NULL,		/* base_init */
-	NULL,		/* base_finalize */
-	(GClassInitFunc) gtk_frame_class_init,
-	NULL,		/* class_finalize */
-	NULL,		/* class_data */
-	sizeof (GtkFrame),
-	0,		/* n_preallocs */
-	(GInstanceInitFunc) gtk_frame_init,
-      };
-
-      frame_type = g_type_register_static (GTK_TYPE_BIN, "GtkFrame",
-					   &frame_info, 0);
-    }
-
-  return frame_type;
-}
+G_DEFINE_TYPE_WITH_CODE (GtkFrame, gtk_frame, GTK_TYPE_BIN,
+			 G_IMPLEMENT_INTERFACE (GTK_TYPE_BUILDABLE,
+						gtk_frame_buildable_init))
 
 static void
 gtk_frame_class_init (GtkFrameClass *class)
@@ -115,8 +96,6 @@ gtk_frame_class_init (GtkFrameClass *class)
   widget_class = GTK_WIDGET_CLASS (class);
   container_class = GTK_CONTAINER_CLASS (class);
 
-  parent_class = g_type_class_peek_parent (class);
-
   gobject_class->set_property = gtk_frame_set_property;
   gobject_class->get_property = gtk_frame_get_property;
 
@@ -126,51 +105,49 @@ gtk_frame_class_init (GtkFrameClass *class)
                                                         P_("Label"),
                                                         P_("Text of the frame's label"),
                                                         NULL,
-                                                        G_PARAM_READABLE |
-							G_PARAM_WRITABLE));
+                                                        GTK_PARAM_READABLE |
+							GTK_PARAM_WRITABLE));
   g_object_class_install_property (gobject_class,
 				   PROP_LABEL_XALIGN,
-				   g_param_spec_float ("label_xalign",
+				   g_param_spec_float ("label-xalign",
 						       P_("Label xalign"),
 						       P_("The horizontal alignment of the label"),
 						       0.0,
 						       1.0,
 						       0.5,
-						       G_PARAM_READABLE |
-						       G_PARAM_WRITABLE));
+						       GTK_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
 				   PROP_LABEL_YALIGN,
-				   g_param_spec_float ("label_yalign",
+				   g_param_spec_float ("label-yalign",
 						       P_("Label yalign"),
 						       P_("The vertical alignment of the label"),
 						       0.0,
 						       1.0,
 						       0.5,
-						       G_PARAM_READABLE |
-						       G_PARAM_WRITABLE));
+						       GTK_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
                                    PROP_SHADOW,
                                    g_param_spec_enum ("shadow", NULL,
                                                       P_("Deprecated property, use shadow_type instead"),
 						      GTK_TYPE_SHADOW_TYPE,
 						      GTK_SHADOW_ETCHED_IN,
-                                                      G_PARAM_READABLE | G_PARAM_WRITABLE));
+                                                      GTK_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
                                    PROP_SHADOW_TYPE,
-                                   g_param_spec_enum ("shadow_type",
+                                   g_param_spec_enum ("shadow-type",
                                                       P_("Frame shadow"),
                                                       P_("Appearance of the frame border"),
 						      GTK_TYPE_SHADOW_TYPE,
 						      GTK_SHADOW_ETCHED_IN,
-                                                      G_PARAM_READABLE | G_PARAM_WRITABLE));
+                                                      GTK_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
                                    PROP_LABEL_WIDGET,
-                                   g_param_spec_object ("label_widget",
+                                   g_param_spec_object ("label-widget",
                                                         P_("Label widget"),
                                                         P_("A widget to display in place of the usual frame label"),
                                                         GTK_TYPE_WIDGET,
-                                                        G_PARAM_READABLE | G_PARAM_WRITABLE));
+                                                        GTK_PARAM_READWRITE));
   
   widget_class->expose_event = gtk_frame_expose;
   widget_class->size_request = gtk_frame_size_request;
@@ -182,6 +159,26 @@ gtk_frame_class_init (GtkFrameClass *class)
   class->compute_child_allocation = gtk_frame_real_compute_child_allocation;
 }
 
+static void
+gtk_frame_buildable_init (GtkBuildableIface *iface)
+{
+  iface->add = gtk_frame_buildable_add;
+}
+
+static void
+gtk_frame_buildable_add (GtkBuildable *buildable,
+			 GtkBuilder   *builder,
+			 GObject      *child,
+			 const gchar  *type)
+{
+  if (type && strcmp (type, "label") == 0)
+    gtk_frame_set_label_widget (GTK_FRAME (buildable), GTK_WIDGET (child));
+  else if (!type)
+    gtk_container_add (GTK_CONTAINER (buildable), GTK_WIDGET (child));
+  else
+    GTK_BUILDER_WARN_INVALID_CHILD_TYPE (GTK_FRAME (buildable), type);
+}
+
 static void
 gtk_frame_init (GtkFrame *frame)
 {
@@ -263,6 +260,15 @@ gtk_frame_get_property (GObject         *object,
     }
 }
 
+/**
+ * gtk_frame_new:
+ * @label: the text to use as the label of the frame
+ * 
+ * Creates a new #GtkFrame, with optional label @label.
+ * If @label is %NULL, the label is omitted.
+ * 
+ * Return value: a new #GtkFrame widget
+ **/
 GtkWidget*
 gtk_frame_new (const gchar *label)
 {
@@ -278,7 +284,7 @@ gtk_frame_remove (GtkContainer *container,
   if (frame->label_widget == child)
     gtk_frame_set_label_widget (frame, NULL);
   else
-    GTK_CONTAINER_CLASS (parent_class)->remove (container, child);
+    GTK_CONTAINER_CLASS (gtk_frame_parent_class)->remove (container, child);
 }
 
 static void
@@ -297,6 +303,14 @@ gtk_frame_forall (GtkContainer *container,
     (* callback) (frame->label_widget, callback_data);
 }
 
+/**
+ * gtk_frame_set_label:
+ * @frame: a #GtkFrame
+ * @label: the text to use as the label of the frame
+ * 
+ * Sets the text of the label. If @label is %NULL,
+ * the current label is removed.
+ **/
 void
 gtk_frame_set_label (GtkFrame *frame,
 		     const gchar *label)
@@ -320,7 +334,7 @@ gtk_frame_set_label (GtkFrame *frame,
  * gtk_frame_get_label:
  * @frame: a #GtkFrame
  * 
- * If the frame's label widget is a #GtkLabel, return the
+ * If the frame's label widget is a #GtkLabel, returns the
  * text in the label widget. (The frame will have a #GtkLabel
  * for the label widget if a non-%NULL argument was passed
  * to gtk_frame_new().)
@@ -346,7 +360,7 @@ gtk_frame_get_label (GtkFrame *frame)
  * @frame: a #GtkFrame
  * @label_widget: the new label widget
  * 
- * Set the label widget for the frame. This is the widget that
+ * Sets the label widget for the frame. This is the widget that
  * will appear embedded in the top edge of the frame as a
  * title.
  **/
@@ -382,7 +396,7 @@ gtk_frame_set_label_widget (GtkFrame  *frame,
     gtk_widget_queue_resize (GTK_WIDGET (frame));
 
   g_object_freeze_notify (G_OBJECT (frame));
-  g_object_notify (G_OBJECT (frame), "label_widget");
+  g_object_notify (G_OBJECT (frame), "label-widget");
   g_object_notify (G_OBJECT (frame), "label");
   g_object_thaw_notify (G_OBJECT (frame));
 }
@@ -404,6 +418,18 @@ gtk_frame_get_label_widget (GtkFrame *frame)
   return frame->label_widget;
 }
 
+/**
+ * gtk_frame_set_label_align:
+ * @frame: a #GtkFrame
+ * @xalign: The position of the label along the top edge
+ *   of the widget. A value of 0.0 represents left alignment;
+ *   1.0 represents right alignment.
+ * @yalign: The y alignment of the label. A value of 0.0 aligns under 
+ *   the frame; 1.0 aligns above the frame.
+ * 
+ * Sets the alignment of the frame widget's label. The
+ * default values for a newly created frame are 0.0 and 0.5.
+ **/
 void
 gtk_frame_set_label_align (GtkFrame *frame,
 			   gfloat    xalign,
@@ -418,13 +444,13 @@ gtk_frame_set_label_align (GtkFrame *frame,
   if (xalign != frame->label_xalign)
     {
       frame->label_xalign = xalign;
-      g_object_notify (G_OBJECT (frame), "label_xalign");
+      g_object_notify (G_OBJECT (frame), "label-xalign");
     }
 
   if (yalign != frame->label_yalign)
     {
       frame->label_yalign = yalign;
-      g_object_notify (G_OBJECT (frame), "label_yalign");
+      g_object_notify (G_OBJECT (frame), "label-yalign");
     }
 
   g_object_thaw_notify (G_OBJECT (frame));
@@ -453,6 +479,13 @@ gtk_frame_get_label_align (GtkFrame *frame,
     *yalign = frame->label_yalign;
 }
 
+/**
+ * gtk_frame_set_shadow_type:
+ * @frame: a #GtkFrame
+ * @type: the new #GtkShadowType
+ * 
+ * Sets the shadow type for @frame.
+ **/
 void
 gtk_frame_set_shadow_type (GtkFrame      *frame,
 			   GtkShadowType  type)
@@ -462,7 +495,7 @@ gtk_frame_set_shadow_type (GtkFrame      *frame,
   if ((GtkShadowType) frame->shadow_type != type)
     {
       frame->shadow_type = type;
-      g_object_notify (G_OBJECT (frame), "shadow_type");
+      g_object_notify (G_OBJECT (frame), "shadow-type");
 
       if (GTK_WIDGET_DRAWABLE (frame))
 	{
@@ -521,14 +554,15 @@ gtk_frame_paint (GtkWidget    *widget,
 	    xalign = 1 - frame->label_xalign;
 
 	  height_extra = MAX (0, child_requisition.height - widget->style->ythickness);
-	  y -= height_extra * (1 - frame->label_yalign);
-	  height += height_extra * (1 - frame->label_yalign);
+	  height_extra *= (1 - frame->label_yalign);
+	  y -= height_extra;
+	  height += height_extra;
 	  
 	  x2 = widget->style->xthickness + (frame->child_allocation.width - child_requisition.width - 2 * LABEL_PAD - 2 * LABEL_SIDE_PAD) * xalign + LABEL_SIDE_PAD;
 
 	  
 	  gtk_paint_shadow_gap (widget->style, widget->window,
-				GTK_STATE_NORMAL, frame->shadow_type,
+				widget->state, frame->shadow_type,
 				area, widget, "frame",
 				x, y, width, height,
 				GTK_POS_TOP, 
@@ -536,7 +570,7 @@ gtk_frame_paint (GtkWidget    *widget,
 	}
        else
 	 gtk_paint_shadow (widget->style, widget->window,
-			   GTK_STATE_NORMAL, frame->shadow_type,
+			   widget->state, frame->shadow_type,
 			   area, widget, "frame",
 			   x, y, width, height);
     }
@@ -550,7 +584,7 @@ gtk_frame_expose (GtkWidget      *widget,
     {
       gtk_frame_paint (widget, &event->area);
 
-      (* GTK_WIDGET_CLASS (parent_class)->expose_event) (widget, event);
+      (* GTK_WIDGET_CLASS (gtk_frame_parent_class)->expose_event) (widget, event);
     }
 
   return FALSE;
@@ -636,7 +670,7 @@ gtk_frame_size_allocate (GtkWidget     *widget,
 	(frame->child_allocation.width - child_requisition.width - 2 * LABEL_PAD - 2 * LABEL_SIDE_PAD) * xalign + LABEL_PAD;
       child_allocation.width = child_requisition.width;
 
-      child_allocation.y = frame->child_allocation.y - child_requisition.height;
+      child_allocation.y = frame->child_allocation.y - MAX (child_requisition.height, widget->style->ythickness);
       child_allocation.height = child_requisition.height;
 
       gtk_widget_size_allocate (frame->label_widget, &child_allocation);
@@ -682,3 +716,6 @@ gtk_frame_real_compute_child_allocation (GtkFrame      *frame,
   child_allocation->x += allocation->x;
   child_allocation->y += allocation->y;
 }
+
+#define __GTK_FRAME_C__
+#include "gtkaliasdef.c"
