@@ -1,5 +1,41 @@
-#ifndef GTK_TEXT_BTREE_H
-#define GTK_TEXT_BTREE_H
+/* GTK - The GIMP Toolkit
+ * gtktextbtree.h Copyright (C) 2000 Red Hat, Inc.
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
+ *
+ * This library is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
+ */
+
+/*
+ * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
+ * file for a list of people on the GTK+ Team.  See the ChangeLog
+ * files for a list of changes.  These files are distributed with
+ * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
+ */
+
+#ifndef __GTK_TEXT_BTREE_H__
+#define __GTK_TEXT_BTREE_H__
+
+#if 0
+#define DEBUG_VALIDATION_AND_SCROLLING
+#endif
+
+#ifdef DEBUG_VALIDATION_AND_SCROLLING
+#define DV(x) (x)
+#else
+#define DV(x)
+#endif
 
 #include <gtk/gtktextbuffer.h>
 #include <gtk/gtktexttag.h>
@@ -8,9 +44,7 @@
 #include <gtk/gtktextsegment.h>
 #include <gtk/gtktextiter.h>
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
+G_BEGIN_DECLS
 
 GtkTextBTree  *_gtk_text_btree_new        (GtkTextTagTable *table,
                                            GtkTextBuffer   *buffer);
@@ -58,7 +92,8 @@ void         _gtk_text_btree_remove_view       (GtkTextBTree      *tree,
                                                 gpointer           view_id);
 void         _gtk_text_btree_invalidate_region (GtkTextBTree      *tree,
                                                 const GtkTextIter *start,
-                                                const GtkTextIter *end);
+                                                const GtkTextIter *end,
+                                                gboolean           cursors_only);
 void         _gtk_text_btree_get_view_size     (GtkTextBTree      *tree,
                                                 gpointer           view_id,
                                                 gint              *width,
@@ -163,6 +198,10 @@ gboolean            _gtk_text_btree_get_selection_bounds    (GtkTextBTree
                                                              GtkTextIter        *end);
 void                _gtk_text_btree_place_cursor            (GtkTextBTree       *tree,
                                                              const GtkTextIter  *where);
+void                _gtk_text_btree_select_range            (GtkTextBTree       *tree,
+                                                             const GtkTextIter  *ins,
+							     const GtkTextIter 
+*bound);
 gboolean            _gtk_text_btree_mark_is_insert          (GtkTextBTree       *tree,
                                                              GtkTextMark        *segment);
 gboolean            _gtk_text_btree_mark_is_selection_bound (GtkTextBTree       *tree,
@@ -182,8 +221,8 @@ struct _GtkTextLineData {
   gpointer view_id;
   GtkTextLineData *next;
   gint height;
-  gint width : 24;
-  gint valid : 8;
+  signed int width : 24;
+  guint valid : 8;		/* Actually a boolean */
 };
 
 /*
@@ -202,6 +241,9 @@ struct _GtkTextLine {
   GtkTextLineSegment *segments; /* First in ordered list of segments
                                  * that make up the line. */
   GtkTextLineData *views;      /* data stored here by views */
+  guchar dir_strong;                /* BiDi algo dir of line */
+  guchar dir_propagated_back;       /* BiDi algo dir of next line */
+  guchar dir_propagated_forward;    /* BiDi algo dir of prev line */
 };
 
 
@@ -275,6 +317,8 @@ GtkTextLine    *    _gtk_text_line_previous_could_contain_tag (GtkTextLine
                                                                GtkTextBTree        *tree,
                                                                GtkTextTag          *tag);
 
+GtkTextLineData    *_gtk_text_line_data_new                   (GtkTextLayout     *layout,
+                                                               GtkTextLine       *line);
 
 /* Debug */
 void _gtk_text_btree_check (GtkTextBTree *tree);
@@ -292,9 +336,11 @@ void _gtk_change_node_toggle_count  (GtkTextBTreeNode   *node,
 void _gtk_text_btree_release_mark_segment (GtkTextBTree       *tree,
                                            GtkTextLineSegment *segment);
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+/* for coordination with the tag table */
+void _gtk_text_btree_notify_will_remove_tag (GtkTextBTree *tree,
+                                             GtkTextTag   *tag);
+
+G_END_DECLS
 
 #endif
 
